#
# i-scream central monitoring system
# http://www.i-scream.org
# Copyright (C) 2000-2002 i-scream
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

# -----------------------------------------------------------
# iscream::XMLParser
# http://www.i-scream.org
#
# iscream::XMLParser decodes a string of XML into a hash in a
# very similar way to the Java version in the i-scream CMS.
#
# $Author: tdb $
# $Id: XMLParser.pm,v 1.3 2004/08/01 10:40:33 tdb Exp $
#------------------------------------------------------------

package iscream::XMLParser;

use strict;
use XML::Parser;

my @tagstack;
my %xmlhash;

sub parse {
    my($xml) = @_;
    %xmlhash = ();

    my $parser = new XML::Parser;
    
    $parser->setHandlers(Char => \&char_handler,
                         Start => \&start_handler,
                         End => \&end_handler);
    
    eval { $parser->parse($xml) };
    
    return ($@, %xmlhash);
}

sub char_handler {
    my ($p, $value) = @_;
    &add_value($value);
}

sub start_handler {
    shift;
    my($tag) = shift;
    push(@tagstack, $tag);
    my($name) = shift;
    my($val) = shift;
    while(defined $name && defined $val) {
       push(@tagstack, "attributes.$name");
       &add_value($val);
       pop(@tagstack);
       $name = shift;
       $val = shift;
    }
}

sub end_handler {
    pop(@tagstack); # safe to not check, parser will do it
}

sub add_value {
    my($value) = @_;
    my($tag) = "";
    foreach my $item (@tagstack) {
        $tag = $tag . $item . ".";
    }
    $tag =~ s/^(.*)\.$/$1/;
    $xmlhash{$tag} = $value;
}

1;
